@@ -45,21 +45,25 @@ def currency_id_list():
         return df
 
 
-def get_valid_currency_list(_date=date.today()):
-    url2 = 'http://www4.bcb.gov.br/Download/fechamento/M{}.csv'
-    url2 = url2.format(_date.strftime('%Y%m%d'))
-    res = requests.get(url2)
+def get_valid_currency_list(_date, n=0):
+    url2 = f'http://www4.bcb.gov.br/Download/fechamento/M{_date:%Y%m%d}.csv'
+    try:
+        res = requests.get(url2)
+    except requests.exceptions.ConnectionError as ex:
+        if n >= 3:
+            raise ex
+        return get_valid_currency_list(_date, n + 1)
     if res.status_code == 200:
         return res
     else:
-        return get_valid_currency_list(_date - timedelta(1))
+        return get_valid_currency_list(_date - timedelta(1), 0)
 
 
 def get_currency_list():
     if CACHE.get('TEMP_FILE_CURRENCY_LIST') is not None:
         return CACHE.get('TEMP_FILE_CURRENCY_LIST')
     else:
-        res = get_valid_currency_list()
+        res = get_valid_currency_list(date.today())
         df = pd.read_csv(StringIO(res.text), delimiter=';')
         df.columns = ['code', 'name', 'symbol', 'country_code', 'country_name',
                       'type', 'exclusion_date']
@@ -112,7 +116,7 @@ def get_symbol(symbol, start_date, end_date):
     return df1
 
 
-def get(symbols, start_date, end_date):
+def get(symbols, start_date, end_date, side='ask', group_by='symbol'):
     if isinstance(symbols, str):
         symbols = [symbols]
     dss = []
@@ -121,6 +125,16 @@ def get(symbols, start_date, end_date):
         if df1 is not None:
             dss.append(df1)
     if len(dss) > 0:
-        return pd.concat(dss, axis=1)
+        df = pd.concat(dss, axis=1)
+        if side in ('bid', 'ask'):
+            dx = df.reorder_levels([1, 0], axis=1).sort_index(axis=1)
+            return dx[side]
+        elif side == 'both':
+            if group_by == 'symbol':
+                return df
+            elif group_by == 'side':
+                return df.reorder_levels([1, 0], axis=1).sort_index(axis=1)
+        else:
+            raise Exception(f'Unknown side value, use: bid, ask, both')
     else:
         return None
